{% extends base_template|default:"handyhelpers/handyhelpers_base.htm" %}
{% load static %}
{% load handyhelper_tags %}

{% block local_head %}
    {% include "handyhelpers/component/table_components.htm" %}
{% endblock %}

{% block content %}
    {% include "handyhelpers/generic/generic_list_content.htm" %}
{% endblock content %}
